# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'database_item.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DatabaseItemUI(object):
    def setupUi(self, DatabaseItemUI):
        DatabaseItemUI.setObjectName("DatabaseItemUI")
        DatabaseItemUI.resize(291, 24)
        self.horizontalLayout = QtWidgets.QHBoxLayout(DatabaseItemUI)
        self.horizontalLayout.setContentsMargins(2, 2, 2, 2)
        self.horizontalLayout.setSpacing(10)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.bus = QtWidgets.QComboBox(DatabaseItemUI)
        self.bus.setObjectName("bus")
        self.horizontalLayout.addWidget(self.bus)
        self.database = QtWidgets.QLabel(DatabaseItemUI)
        self.database.setObjectName("database")
        self.horizontalLayout.addWidget(self.database)
        self.horizontalLayout.setStretch(1, 1)

        self.retranslateUi(DatabaseItemUI)
        QtCore.QMetaObject.connectSlotsByName(DatabaseItemUI)

    def retranslateUi(self, DatabaseItemUI):
        _translate = QtCore.QCoreApplication.translate
        DatabaseItemUI.setWindowTitle(_translate("DatabaseItemUI", "Form"))
        self.database.setText(_translate("DatabaseItemUI", "TextLabel"))
