# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'define_channel_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ComputedChannel(object):
    def setupUi(self, ComputedChannel):
        ComputedChannel.setObjectName("ComputedChannel")
        ComputedChannel.resize(523, 395)
        ComputedChannel.setMaximumSize(QtCore.QSize(16777215, 400))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/plus.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        ComputedChannel.setWindowIcon(icon)
        self.gridLayout = QtWidgets.QGridLayout(ComputedChannel)
        self.gridLayout.setObjectName("gridLayout")
        self.apply_btn = QtWidgets.QPushButton(ComputedChannel)
        self.apply_btn.setObjectName("apply_btn")
        self.gridLayout.addWidget(self.apply_btn, 1, 1, 1, 1)
        self.cancel_btn = QtWidgets.QPushButton(ComputedChannel)
        self.cancel_btn.setObjectName("cancel_btn")
        self.gridLayout.addWidget(self.cancel_btn, 1, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 0, 1, 1)
        self.tabs = QtWidgets.QTabWidget(ComputedChannel)
        self.tabs.setObjectName("tabs")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.computation_grid_layout = QtWidgets.QGridLayout(self.tab)
        self.computation_grid_layout.setObjectName("computation_grid_layout")
        self.op = QtWidgets.QComboBox(self.tab)
        self.op.setObjectName("op")
        self.computation_grid_layout.addWidget(self.op, 1, 0, 1, 1)
        self.operand2 = QtWidgets.QComboBox(self.tab)
        self.operand2.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.operand2.setObjectName("operand2")
        self.computation_grid_layout.addWidget(self.operand2, 2, 0, 1, 2)
        spacerItem1 = QtWidgets.QSpacerItem(29, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.computation_grid_layout.addItem(spacerItem1, 1, 1, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.computation_grid_layout.addItem(spacerItem2, 5, 0, 1, 1)
        self.operand1 = QtWidgets.QComboBox(self.tab)
        self.operand1.setMinimumSize(QtCore.QSize(380, 20))
        self.operand1.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.operand1.setObjectName("operand1")
        self.computation_grid_layout.addWidget(self.operand1, 0, 0, 1, 2)
        spacerItem3 = QtWidgets.QSpacerItem(92, 18, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.computation_grid_layout.addItem(spacerItem3, 4, 2, 1, 1)
        self.unit = QtWidgets.QLineEdit(self.tab)
        self.unit.setObjectName("unit")
        self.computation_grid_layout.addWidget(self.unit, 4, 0, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.computation_grid_layout.addItem(spacerItem4, 4, 3, 1, 1)
        self.name = QtWidgets.QLineEdit(self.tab)
        self.name.setInputMask("")
        self.name.setText("")
        self.name.setObjectName("name")
        self.computation_grid_layout.addWidget(self.name, 3, 0, 1, 4)
        self.computation_grid_layout.setColumnStretch(2, 1)
        self.computation_grid_layout.setColumnStretch(3, 1)
        self.tabs.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.function_unit = QtWidgets.QLineEdit(self.tab_2)
        self.function_unit.setObjectName("function_unit")
        self.gridLayout_4.addWidget(self.function_unit, 3, 0, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(92, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem5, 1, 2, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(184, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem6, 3, 1, 1, 1)
        self.channel = QtWidgets.QComboBox(self.tab_2)
        self.channel.setObjectName("channel")
        self.gridLayout_4.addWidget(self.channel, 1, 0, 1, 2)
        self.function_name = QtWidgets.QLineEdit(self.tab_2)
        self.function_name.setObjectName("function_name")
        self.gridLayout_4.addWidget(self.function_name, 2, 0, 1, 3)
        self.function = QtWidgets.QComboBox(self.tab_2)
        self.function.setMinimumSize(QtCore.QSize(380, 20))
        self.function.setObjectName("function")
        self.gridLayout_4.addWidget(self.function, 0, 0, 1, 2)
        spacerItem7 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem7, 4, 0, 1, 1)
        self.tabs.addTab(self.tab_2, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab_3)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.expression_name = QtWidgets.QLineEdit(self.tab_3)
        self.expression_name.setObjectName("expression_name")
        self.gridLayout_2.addWidget(self.expression_name, 2, 0, 1, 2)
        self.expression_unit = QtWidgets.QLineEdit(self.tab_3)
        self.expression_unit.setObjectName("expression_unit")
        self.gridLayout_2.addWidget(self.expression_unit, 3, 0, 1, 1)
        self.label = QtWidgets.QLabel(self.tab_3)
        self.label.setTextFormat(QtCore.Qt.RichText)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 2)
        spacerItem8 = QtWidgets.QSpacerItem(234, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem8, 3, 1, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem9, 4, 0, 1, 1)
        self.expression = QtWidgets.QPlainTextEdit(self.tab_3)
        self.expression.setObjectName("expression")
        self.gridLayout_2.addWidget(self.expression, 1, 0, 1, 2)
        self.tabs.addTab(self.tab_3, "")
        self.gridLayout.addWidget(self.tabs, 0, 0, 1, 3)

        self.retranslateUi(ComputedChannel)
        self.tabs.setCurrentIndex(2)
        QtCore.QMetaObject.connectSlotsByName(ComputedChannel)

    def retranslateUi(self, ComputedChannel):
        _translate = QtCore.QCoreApplication.translate
        ComputedChannel.setWindowTitle(_translate("ComputedChannel", "Define new channel"))
        self.apply_btn.setText(_translate("ComputedChannel", "Apply"))
        self.cancel_btn.setText(_translate("ComputedChannel", "Cancel"))
        self.unit.setPlaceholderText(_translate("ComputedChannel", "channel unit"))
        self.name.setPlaceholderText(_translate("ComputedChannel", "channel name"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab), _translate("ComputedChannel", "Simple computation"))
        self.function_unit.setPlaceholderText(_translate("ComputedChannel", "channel unit"))
        self.function_name.setPlaceholderText(_translate("ComputedChannel", "channel name"))
        self.function.setToolTip(_translate("ComputedChannel", "see numpy documentation"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_2), _translate("ComputedChannel", "Function"))
        self.expression_name.setPlaceholderText(_translate("ComputedChannel", "channel name"))
        self.expression_unit.setPlaceholderText(_translate("ComputedChannel", "channel unit"))
        self.label.setText(_translate("ComputedChannel", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'MS Shell Dlg 2\'; font-size:8pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The expression is evaluated using the <span style=\" font-style:italic;\">numexpr </span>library. Have a look at the <a href=\"https://numexpr.readthedocs.io/projects/NumExpr3/en/latest/user_guide.html\"><span style=\" text-decoration: underline; color:#0000ff;\">Numexpr documentation</span></a> for supported operators and function.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The signal names must be enclosed by double curly braces: <span style=\" font-weight:600; color:#ff0000;\">{{</span>Signal_name<span style=\" font-weight:600; color:#ff0000;\">}}</span></p></body></html>"))
        self.expression.setPlaceholderText(_translate("ComputedChannel", "enter your expression here. Example: ({{Sig1}} + {{Sig2}}) / ({{Sig3}} - {{Sig4}}) + 7.8"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_3), _translate("ComputedChannel", "Expression"))
from . import resource_rc
